@@ -3,7 +3,7 @@
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 
 from typing import Optional
 from typing import Sequence
@@ -35,6 +35,44 @@ from . import names
 from . import util
 
 
+def infer_type_from_right_hand_nameexpr(
+    api: SemanticAnalyzerPluginInterface,
+    stmt: AssignmentStmt,
+    node: Var,
+    left_hand_explicit_type: Optional[ProperType],
+    infer_from_right_side: RefExpr,
+) -> Optional[ProperType]:
+
+    type_id = names.type_id_for_callee(infer_from_right_side)
+
+    if type_id is None:
+        return None
+    elif type_id is names.COLUMN:
+        python_type_for_type = _infer_type_from_decl_column(
+            api, stmt, node, left_hand_explicit_type
+        )
+    elif type_id is names.RELATIONSHIP:
+        python_type_for_type = _infer_type_from_relationship(
+            api, stmt, node, left_hand_explicit_type
+        )
+    elif type_id is names.COLUMN_PROPERTY:
+        python_type_for_type = _infer_type_from_decl_column_property(
+            api, stmt, node, left_hand_explicit_type
+        )
+    elif type_id is names.SYNONYM_PROPERTY:
+        python_type_for_type = infer_type_from_left_hand_type_only(
+            api, node, left_hand_explicit_type
+        )
+    elif type_id is names.COMPOSITE_PROPERTY:
+        python_type_for_type = _infer_type_from_decl_composite_property(
+            api, stmt, node, left_hand_explicit_type
+        )
+    else:
+        return None
+
+    return python_type_for_type
+
+
 def _infer_type_from_relationship(
     api: SemanticAnalyzerPluginInterface,
     stmt: AssignmentStmt,
@@ -90,8 +128,8 @@ def _infer_type_from_relationship(
     # string expression
     # isinstance(target_cls_arg, StrExpr)
 
-    uselist_arg = util._get_callexpr_kwarg(stmt.rvalue, "uselist")
-    collection_cls_arg: Optional[Expression] = util._get_callexpr_kwarg(
+    uselist_arg = util.get_callexpr_kwarg(stmt.rvalue, "uselist")
+    collection_cls_arg: Optional[Expression] = util.get_callexpr_kwarg(
         stmt.rvalue, "collection_class"
     )
     type_is_a_collection = False
@@ -99,7 +137,7 @@ def _infer_type_from_relationship(
     # this can be used to determine Optional for a many-to-one
     # in the same way nullable=False could be used, if we start supporting
     # that.
-    # innerjoin_arg = _get_callexpr_kwarg(stmt.rvalue, "innerjoin")
+    # innerjoin_arg = util.get_callexpr_kwarg(stmt.rvalue, "innerjoin")
 
     if (
         uselist_arg is not None
@@ -180,7 +218,7 @@ def _infer_type_from_relationship(
             util.fail(api, msg.format(node.name), node)
 
     if python_type_for_type is None:
-        return _infer_type_from_left_hand_type_only(
+        return infer_type_from_left_hand_type_only(
             api, node, left_hand_explicit_type
         )
     elif left_hand_explicit_type is not None:
@@ -222,7 +260,7 @@ def _infer_type_from_decl_composite_property(
         python_type_for_type = None
 
     if python_type_for_type is None:
-        return _infer_type_from_left_hand_type_only(
+        return infer_type_from_left_hand_type_only(
             api, node, left_hand_explicit_type
         )
     elif left_hand_explicit_type is not None:
@@ -246,19 +284,38 @@ def _infer_type_from_decl_column_property(
 
     """
     assert isinstance(stmt.rvalue, CallExpr)
-    first_prop_arg = stmt.rvalue.args[0]
 
-    if isinstance(first_prop_arg, CallExpr):
-        type_id = names._type_id_for_callee(first_prop_arg.callee)
+    if stmt.rvalue.args:
+        first_prop_arg = stmt.rvalue.args[0]
 
-        # look for column_property() / deferred() etc with Column as first
-        # argument
-        if type_id is names.COLUMN:
+        if isinstance(first_prop_arg, CallExpr):
+            type_id = names.type_id_for_callee(first_prop_arg.callee)
+
+            # look for column_property() / deferred() etc with Column as first
+            # argument
+            if type_id is names.COLUMN:
+                return _infer_type_from_decl_column(
+                    api,
+                    stmt,
+                    node,
+                    left_hand_explicit_type,
+                    right_hand_expression=first_prop_arg,
+                )
+
+    if isinstance(stmt.rvalue, CallExpr):
+        type_id = names.type_id_for_callee(stmt.rvalue.callee)
+        # this is probably not strictly necessary as we have to use the left
+        # hand type for query expression in any case.  any other no-arg
+        # column prop objects would go here also
+        if type_id is names.QUERY_EXPRESSION:
             return _infer_type_from_decl_column(
-                api, stmt, node, left_hand_explicit_type, first_prop_arg
+                api,
+                stmt,
+                node,
+                left_hand_explicit_type,
             )
 
-    return _infer_type_from_left_hand_type_only(
+    return infer_type_from_left_hand_type_only(
         api, node, left_hand_explicit_type
     )
 
@@ -268,7 +325,7 @@ def _infer_type_from_decl_column(
     stmt: AssignmentStmt,
     node: Var,
     left_hand_explicit_type: Optional[ProperType],
-    right_hand_expression: CallExpr,
+    right_hand_expression: Optional[CallExpr] = None,
 ) -> Optional[ProperType]:
     """Infer the type of mapping from a Column.
 
@@ -305,6 +362,12 @@ def _infer_type_from_decl_column(
 
     callee = None
 
+    if right_hand_expression is None:
+        if not isinstance(stmt.rvalue, CallExpr):
+            return None
+
+        right_hand_expression = stmt.rvalue
+
     for column_arg in right_hand_expression.args[0:2]:
         if isinstance(column_arg, CallExpr):
             if isinstance(column_arg.callee, RefExpr):
@@ -330,10 +393,10 @@ def _infer_type_from_decl_column(
     if callee is None:
         return None
 
-    if isinstance(callee.node, TypeInfo) and names._mro_has_id(
+    if isinstance(callee.node, TypeInfo) and names.mro_has_id(
         callee.node.mro, names.TYPEENGINE
     ):
-        python_type_for_type = _extract_python_type_from_typeengine(
+        python_type_for_type = extract_python_type_from_typeengine(
             api, callee.node, type_args
         )
 
@@ -348,7 +411,7 @@ def _infer_type_from_decl_column(
     else:
         # it's not TypeEngine, it's typically implicitly typed
         # like ForeignKey.  we can't infer from the right side.
-        return _infer_type_from_left_hand_type_only(
+        return infer_type_from_left_hand_type_only(
             api, node, left_hand_explicit_type
         )
 
@@ -424,7 +487,7 @@ def _infer_collection_type_from_left_and_inferred_right(
     )
 
 
-def _infer_type_from_left_hand_type_only(
+def infer_type_from_left_hand_type_only(
     api: SemanticAnalyzerPluginInterface,
     node: Var,
     left_hand_explicit_type: Optional[ProperType],
@@ -451,7 +514,7 @@ def _infer_type_from_left_hand_type_only(
         return left_hand_explicit_type
 
 
-def _extract_python_type_from_typeengine(
+def extract_python_type_from_typeengine(
     api: SemanticAnalyzerPluginInterface,
     node: TypeInfo,
     type_args: Sequence[Expression],
